@@ -27,17 +27,18 @@ has 'atoms' => (
     lazy => 1,
 );
 
-sub tmax {
+#sub tmax {
     # not the best implementation! what about atoms without coords?
     # comparing first and last?  really?!  
     # use coords to calculate tmax,which may be annoying if just interested in charges
-    my $self = shift;
-    return (0) unless $self->count_atoms;
-    my $t0   = $self->get_atoms(0)->count_coords;
-    my $tn   = $self->get_atoms($self->count_atoms - 1)->count_coords;
-    croak "first and last atoms no tmax" unless($t0 == $tn);
-    return ($t0-1);
-}
+#    my $self = shift;
+#    return (0) unless $self->count_atoms;
+  
+#    my $t0   = $self->get_atoms(0)->count_coords;
+#    my $tn   = $self->get_atoms($self->count_atoms - 1)->count_coords;
+#    croak "first and last atoms no tmax" unless($t0 == $tn);
+#    return ($t0-1);
+#}
 
 sub dipole {
     my $self = shift;
@@ -174,7 +175,6 @@ sub translate {
 }
 
 sub rotate {
-
     #rotate about origin. having origin allows rotation of subgroup
     #without having to translate everything.
     my $self = shift;
@@ -194,6 +194,71 @@ sub rotate {
     $atoms[$_]->set_coords( $tf, $rcor[$_] + $orig ) foreach 0 .. $#rcor;
 }
 
+sub print_xyz_ts {
+  _print_ts('print_xyz',@_);
+}
+
+sub print_pdb_ts {
+  _print_ts('print_pdb',@_);
+}
+
+sub _print_ts {
+    #use one sub for xyz_ts and pdb_ts writing
+    my $print_method = shift;
+    # two args: \@ts, optional filename
+    my $self = shift;
+    my $ts   = shift;
+    unless(defined($ts)) {
+      croak "must pass arrayref containing ts";
+    }
+    my @ts   = @$ts;
+    unless(scalar(@ts)) {
+      croak "must pass array with atleast one t";
+    }
+    my $tmax = $self->tmax;
+    my $nt = grep {$_ > $tmax} @ts;
+    croak "$nt ts out of bounds" if $nt;
+  
+    my $tnow = $self->what_time;
+    # take the first out of the loop to setup fh
+    $self->gt(shift @ts);
+    my $fh = $self->$print_method(@_);
+ 
+    foreach my $t (@ts){
+      $self->gt($t);
+      $fh = $self->$print_method($fh);
+    }
+    # return to original t
+    $self->gt($tnow);
+}
+
+sub bin_this{
+  #return binned $_->method, what if can't be binned.  bah.  
+  my $self   = shift;
+  my $method = shift;
+  my @ts   = map{$_->$method} $self->all_atoms;
+  my %bin;
+  $bin{$_}++ foreach @ts;
+  return (\%bin);
+}
+
+sub tmax {
+    # still not the best implementation! what about atoms without coords?
+    my $self = shift;
+    my $tbin = $self->bin_this('count_coords');
+    my @ts   = keys(%$tbin);
+    croak "tmax differences within group" if (scalar(@ts)> 1);
+    $ts[0] ? return $ts[0]-1 : return 0;
+}
+
+sub what_time {
+    my $self = shift;
+    my $tbin = $self->bin_this('t');
+    my @ts   = keys(%$tbin);
+    croak "t differences within group" if (scalar(@ts)> 1);
+    return $ts[0];
+}
+
 sub print_xyz {
     my $self = shift;
     my $fh = _open_file_unless_fh(shift);
@@ -278,7 +343,7 @@ HackaMol::AtomGroupRole - Role for a group of atoms
 
 =head1 VERSION
 
-version 0.00_09
+version 0.00_10
 
 =head1 SYNOPSIS
 
@@ -407,7 +472,7 @@ by Z and generates something like OH2 for water or O2H2 for peroxide.
 
 =head2 tmax
 
-return (count_coords-1) of first atom; checks that count_coords is same for first and last atoms
+return (count_coords-1) if > 0; return 0 otherwise; croaks if not all atoms share the same tmax.
 
 =head2 translate
 
@@ -430,10 +495,30 @@ optional argument: filename or filehandle.  with no argument, prints xyz formatt
 a filename and an xyz file with that name will be written or overwritten (with warning). pass filehandle 
 for continuous writing to an open filehandle.
 
+=head2 print_xyz_ts
+
+argument: array_ref containing the values of t to be used for printing.  
+optional argument: filename or filehandle for writing out to file. For example,
+
+  $mol->print_xyz_ts([0 .. 3, 8, 4], 'fun.xyz');
+
+will write the coordinates for all group atoms at t=0,1,2,3,8,4 to a file, in
+that order.
+
 =head2 print_pdb
 
 same as print_xyz, but for pdb formatted output
 
+=head2 print_pdb_ts
+
+same as print_xyz_ts, but for pdb formatted output
+
+=head2 bin_this
+
+argument: Str , return hash_ref of binned $self->Str.
+
+  $hash_ref{$_}++ foreach ( map {$_->$Str} $self->all_atoms );
+
 =head1 ARRAY METHODS
 
 =head2 push_atoms, get_atoms, set_atoms, all_atoms, count_atoms, clear_atoms
